@@ -17,6 +17,20 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/**
+ * SECTION:element-mad
+ * @see_also: lame
+ *
+ * MP3 audio decoder.
+ *
+ * <refsect2>
+ * <title>Example pipelines</title>
+ * |[
+ * gst-launch filesrc location=music.mp3 ! mad ! audioconvert ! audioresample ! autoaudiosink
+ * ]| Decode the mp3 file and play
+ * </refsect2>
+ */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -95,8 +109,10 @@ static GstFlowReturn gst_mad_chain (GstPad * pad, GstBuffer * buffer);
 static GstStateChangeReturn gst_mad_change_state (GstElement * element,
     GstStateChange transition);
 
+#ifndef GST_DISABLE_INDEX
 static void gst_mad_set_index (GstElement * element, GstIndex * index);
 static GstIndex *gst_mad_get_index (GstElement * element);
+#endif
 
 static void
 _do_init (GType type)
@@ -178,6 +194,7 @@ gst_mad_base_init (gpointer g_class)
       gst_static_pad_template_get (&mad_src_template_factory));
   gst_element_class_set_details (element_class, &gst_mad_details);
 }
+
 static void
 gst_mad_class_init (GstMadClass * klass)
 {
@@ -194,8 +211,10 @@ gst_mad_class_init (GstMadClass * klass)
   gobject_class->dispose = gst_mad_dispose;
 
   gstelement_class->change_state = gst_mad_change_state;
+#ifndef GST_DISABLE_INDEX
   gstelement_class->set_index = gst_mad_set_index;
   gstelement_class->get_index = gst_mad_get_index;
+#endif
 
   /* init properties */
   /* currently, string representations are used, we might want to change that */
@@ -279,14 +298,21 @@ gst_mad_dispose (GObject * object)
 {
   GstMad *mad = GST_MAD (object);
 
+#ifndef GST_DISABLE_INDEX
   gst_mad_set_index (GST_ELEMENT (object), NULL);
+#endif
 
   g_free (mad->tempbuffer);
   mad->tempbuffer = NULL;
 
+  g_list_foreach (mad->pending_events, (GFunc) gst_mini_object_unref, NULL);
+  g_list_free (mad->pending_events);
+  mad->pending_events = NULL;
+
   G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
+#ifndef GST_DISABLE_INDEX
 static void
 gst_mad_set_index (GstElement * element, GstIndex * index)
 {
@@ -305,6 +331,7 @@ gst_mad_get_index (GstElement * element)
 
   return mad->index;
 }
+#endif
 
 static gboolean
 gst_mad_convert_sink (GstPad * pad, GstFormat src_format, gint64 src_value,
@@ -380,7 +407,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
 
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
-  bytes_per_sample = MAD_NCHANNELS (&mad->frame.header) * 4;
+  bytes_per_sample = mad->channels * 4;
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -392,7 +419,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           break;
         case GST_FORMAT_TIME:
         {
-          gint byterate = bytes_per_sample * mad->frame.header.samplerate;
+          gint byterate = bytes_per_sample * mad->rate;
 
           if (byterate == 0)
             return FALSE;
@@ -410,10 +437,10 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           *dest_value = src_value * bytes_per_sample;
           break;
         case GST_FORMAT_TIME:
-          if (mad->frame.header.samplerate == 0)
+          if (mad->rate == 0)
             return FALSE;
           *dest_value = gst_util_uint64_scale_int (src_value, GST_SECOND,
-              mad->frame.header.samplerate);
+              mad->rate);
           break;
         default:
           res = FALSE;
@@ -426,7 +453,7 @@ gst_mad_convert_src (GstPad * pad, GstFormat src_format, gint64 src_value,
           /* fallthrough */
         case GST_FORMAT_DEFAULT:
           *dest_value = gst_util_uint64_scale_int (src_value,
-              scale * mad->frame.header.samplerate, GST_SECOND);
+              scale * mad->rate, GST_SECOND);
           break;
         default:
           res = FALSE;
@@ -588,6 +615,7 @@ error:
   return FALSE;
 }
 
+#ifndef GST_DISABLE_INDEX
 static gboolean
 index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
 {
@@ -661,6 +689,7 @@ index_seek (GstMad * mad, GstPad * pad, GstEvent * event)
 
   return FALSE;
 }
+#endif
 
 static gboolean
 normal_seek (GstMad * mad, GstPad * pad, GstEvent * event)
@@ -782,23 +811,24 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
   mad = GST_MAD (GST_PAD_PARENT (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-      /* the all-formats seek logic */
     case GST_EVENT_SEEK:
+      /* the all-formats seek logic, ref the event, we need it later */
       gst_event_ref (event);
-      if (!(res = gst_pad_event_default (pad, event))) {
+      if (!(res = gst_pad_push_event (mad->sinkpad, event))) {
+#ifndef GST_DISABLE_INDEX
         if (mad->index)
           res = index_seek (mad, pad, event);
         else
+#endif
           res = normal_seek (mad, pad, event);
       }
+      gst_event_unref (event);
       break;
-
     default:
-      res = FALSE;
+      res = gst_pad_push_event (mad->sinkpad, event);
       break;
   }
 
-  gst_event_unref (event);
   return res;
 }
 
@@ -846,6 +876,7 @@ gst_mad_set_property (GObject * object, guint prop_id,
       break;
   }
 }
+
 static void
 gst_mad_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec)
@@ -984,11 +1015,18 @@ gst_mad_sink_event (GstPad * pad, GstEvent * event)
       mad->tempsize = 0;
       mad_frame_mute (&mad->frame);
       mad_synth_mute (&mad->synth);
+    case GST_EVENT_FLUSH_START:
       result = gst_pad_event_default (pad, event);
 
       break;
     default:
-      result = gst_pad_event_default (pad, event);
+      if (mad->restart) {
+        /* Cache all other events if we still have to send a NEWSEGMENT */
+        mad->pending_events = g_list_append (mad->pending_events, event);
+        result = TRUE;
+      } else {
+        result = gst_pad_event_default (pad, event);
+      }
       break;
   }
   return result;
@@ -1250,6 +1288,7 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
   guint8 *data;
   glong size, tempsize;
   gboolean new_pts = FALSE;
+  gboolean discont;
   GstClockTime timestamp;
   GstFlowReturn result = GST_FLOW_OK;
 
@@ -1261,6 +1300,9 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     GST_DEBUG ("mad restarted");
   }
 
+  /* take discont flag */
+  discont = GST_BUFFER_IS_DISCONT (buffer);
+
   timestamp = GST_BUFFER_TIMESTAMP (buffer);
   GST_DEBUG ("mad in timestamp %" GST_TIME_FORMAT, GST_TIME_ARGS (timestamp));
 
@@ -1296,6 +1338,10 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
     gint tocopy;
     guchar *mad_input_buffer;   /* convenience pointer to tempbuffer */
 
+    if (mad->tempsize == 0 && discont) {
+      mad->discont = TRUE;
+      discont = FALSE;
+    }
     tocopy =
         MIN (MAD_BUFFER_MDLEN, MIN (size,
             MAD_BUFFER_MDLEN * 3 - mad->tempsize));
@@ -1409,7 +1455,12 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               } else {
                 mad->tags = gst_tag_list_copy (list);
               }
-              gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
+              if (mad->need_newsegment)
+                mad->pending_events =
+                    g_list_append (mad->pending_events,
+                    gst_event_new_tag (list));
+              else
+                gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
             }
           }
         }
@@ -1449,7 +1500,12 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               GST_TAG_BITRATE, bitrate, NULL);
           gst_element_post_message (GST_ELEMENT (mad),
               gst_message_new_tag (GST_OBJECT (mad), gst_tag_list_copy (list)));
-          gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
+
+          if (mad->need_newsegment)
+            mad->pending_events =
+                g_list_append (mad->pending_events, gst_event_new_tag (list));
+          else
+            gst_pad_push_event (mad->srcpad, gst_event_new_tag (list));
 
           goto next_no_samples;
         }
@@ -1461,9 +1517,8 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
       nsamples = MAD_NSBSAMPLES (&mad->frame.header) *
           (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);
 
-      if (mad->frame.header.samplerate == 0) {
-        g_warning
-            ("mad->frame.header.samplerate is 0; timestamps cannot be calculated");
+      if (mad->rate == 0) {
+        g_warning ("mad->rate is 0; timestamps cannot be calculated");
       } else {
         /* if we have a pending timestamp, we can use it now to calculate the sample offset */
         if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
@@ -1510,12 +1565,14 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
             GST_SECOND, mad->rate) - time_offset;
       }
 
+#ifndef GST_DISABLE_INDEX
       if (mad->index) {
         guint64 x_bytes = mad->base_byte_offset + mad->bytes_consumed;
 
         gst_index_add_association (mad->index, mad->index_id, 0,
             GST_FORMAT_BYTES, x_bytes, GST_FORMAT_TIME, time_offset, NULL);
       }
+#endif
 
       if (mad->segment_start <= (time_offset ==
               GST_CLOCK_TIME_NONE ? 0 : time_offset)) {
@@ -1542,6 +1599,16 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
           mad->need_newsegment = FALSE;
         }
 
+        if (mad->pending_events) {
+          GList *l;
+
+          for (l = mad->pending_events; l != NULL; l = l->next) {
+            gst_pad_push_event (mad->srcpad, GST_EVENT (l->data));
+          }
+          g_list_free (mad->pending_events);
+          mad->pending_events = NULL;
+        }
+
         /* will attach the caps to the buffer */
         result =
             gst_pad_alloc_buffer_and_set_caps (mad->srcpad, 0,
@@ -1566,6 +1633,7 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
         GST_BUFFER_TIMESTAMP (outbuffer) = time_offset;
         GST_BUFFER_DURATION (outbuffer) = time_duration;
         GST_BUFFER_OFFSET (outbuffer) = mad->total_samples;
+        GST_BUFFER_OFFSET_END (outbuffer) = mad->total_samples + nsamples;
 
         /* output sample(s) in 16-bit signed native-endian PCM */
         if (mad->channels == 1) {
@@ -1589,6 +1657,13 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
               "pushing buffer, off=%" G_GUINT64_FORMAT ", ts=%" GST_TIME_FORMAT,
               GST_BUFFER_OFFSET (outbuffer),
               GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
+
+          /* apply discont */
+          if (mad->discont) {
+            GST_BUFFER_FLAG_SET (outbuffer, GST_BUFFER_FLAG_DISCONT);
+            mad->discont = FALSE;
+          }
+
           mad->segment.last_stop = GST_BUFFER_TIMESTAMP (outbuffer);
           result = gst_pad_push (mad->srcpad, outbuffer);
           if (result != GST_FLOW_OK) {
@@ -1612,6 +1687,10 @@ gst_mad_chain (GstPad * pad, GstBuffer * buffer)
         mad->bytes_consumed = 0;
       }
       tempsize = 0;
+      if (discont) {
+        mad->discont = TRUE;
+        discont = FALSE;
+      }
 
       if (gst_mad_check_restart (mad)) {
         goto end;
@@ -1663,6 +1742,7 @@ gst_mad_change_state (GstElement * element, GstStateChange transition)
       mad_frame_init (&mad->frame);
       mad_synth_init (&mad->synth);
       mad->tempsize = 0;
+      mad->discont = TRUE;
       mad->total_samples = 0;
       mad->rate = 0;
       mad->channels = 0;
