@@ -32,8 +32,6 @@
  */
 #define PTM_DISCONT_ADJUST (0.3 * GST_SECOND)
 #define INITIAL_END_PTM (-1)
-#define MAX_GAP ( 3 * GST_SECOND / 2 )
-#define MAX_GAP_TOLERANCE ( GST_SECOND / 20 )
 
 GST_DEBUG_CATEGORY_STATIC (gstdvddemux_debug);
 #define GST_CAT_DEFAULT (gstdvddemux_debug)
@@ -43,6 +41,15 @@ GST_DEBUG_CATEGORY_STATIC (gstdvddemux_debug);
 #define DEMUX_CLASS(o)  GST_MPEG_DEMUX_CLASS (G_OBJECT_GET_CLASS (o))
 #define CLASS(o)  GST_DVD_DEMUX_CLASS (G_OBJECT_GET_CLASS (o))
 
+
+/* Element factory information */
+static GstElementDetails dvd_demux_details = {
+  "DVD Demuxer",
+  "Codec/Demuxer",
+  "Demultiplexes DVD (VOB) MPEG2 streams",
+  "Martin Soto <martinsoto@users.sourceforge.net>"
+};
+
 /* DVDDemux signals and args */
 enum
 {
@@ -127,6 +134,11 @@ GST_STATIC_PAD_TEMPLATE ("current_subpicture",
 GST_BOILERPLATE_FULL (GstDVDDemux, gst_dvd_demux, GstMPEGDemux,
     GST_TYPE_MPEG_DEMUX, _do_init);
 
+static void gst_dvd_demux_class_init (GstDVDDemuxClass * klass);
+static void gst_dvd_demux_base_init (gpointer klass);
+static void gst_dvd_demux_init (GstDVDDemux * dvd_demux,
+    GstDVDDemuxClass * klass);
+
 static gboolean gst_dvd_demux_process_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
 static gboolean gst_dvd_demux_parse_packhead (GstMPEGParse * mpeg_parse,
@@ -188,8 +200,8 @@ gst_dvd_demux_base_init (gpointer klass)
   mpeg_parse_class->parse_packhead = gst_dvd_demux_parse_packhead;
 
   /* sink pad */
-  gst_element_class_add_static_pad_template (element_class,
-      &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
 
   demux_class->audio_template = gst_static_pad_template_get (&audio_template);
 
@@ -215,10 +227,7 @@ gst_dvd_demux_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       dvd_demux_class->cur_subpicture_template);
 
-  gst_element_class_set_details_simple (element_class, "DVD Demuxer",
-      "Codec/Demuxer",
-      "Demultiplexes DVD (VOB) MPEG2 streams",
-      "Martin Soto <martinsoto@users.sourceforge.net>");
+  gst_element_class_set_details (element_class, &dvd_demux_details);
 }
 
 
@@ -226,11 +235,13 @@ static void
 gst_dvd_demux_class_init (GstDVDDemuxClass * klass)
 {
   GstElementClass *gstelement_class;
+  GstMPEGParseClass *mpeg_parse_class;
   GstMPEGDemuxClass *mpeg_demux_class;
 
   parent_class = g_type_class_peek_parent (klass);
 
   gstelement_class = (GstElementClass *) klass;
+  mpeg_parse_class = (GstMPEGParseClass *) klass;
   mpeg_demux_class = (GstMPEGDemuxClass *) klass;
 
   gstelement_class->change_state = gst_dvd_demux_change_state;
@@ -422,22 +433,16 @@ gst_dvd_demux_handle_dvd_event (GstDVDDemux * dvd_demux, GstEvent * event)
     }
 
     /* subtitle */
-    for (n = 0; n < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; n++) {
-      t = g_strdup_printf ("subtitle-%d-language", n);
+    for (;;) {
+      t = g_strdup_printf ("subtitle-%d-language", num_substreams);
       if (!gst_structure_get_value (structure, t)) {
         g_free (t);
-        continue;
+        break;
       }
       g_free (t);
-      num_substreams = n + 1;
-    }
-
-    /* now we have a maximum,
-     * and can also fill empty slots in case of cranky DVD */
-    for (n = 0; n < num_substreams; n++)
       CLASS (dvd_demux)->get_subpicture_stream (mpeg_demux,
-          n, GST_DVD_DEMUX_SUBP_DVD, NULL);
-
+          num_substreams++, GST_DVD_DEMUX_SUBP_DVD, NULL);
+    }
     GST_DEBUG_OBJECT (dvd_demux,
         "Created 1 video stream, %d audio streams and %d subpicture streams "
         "based on DVD lang codes event; now signalling no-more-pads",
@@ -447,8 +452,8 @@ gst_dvd_demux_handle_dvd_event (GstDVDDemux * dvd_demux, GstEvent * event)
     gst_element_no_more_pads (GST_ELEMENT (dvd_demux));
 
     /* Keep video/audio/subtitle pads within 1/2 sec of the SCR */
-    mpeg_demux->max_gap = MAX_GAP;
-    mpeg_demux->max_gap_tolerance = MAX_GAP_TOLERANCE;
+    mpeg_demux->max_gap = GST_SECOND / 2;
+    mpeg_demux->max_gap_tolerance = GST_SECOND / 20;
   } else {
     GST_DEBUG_OBJECT (dvd_demux, "dvddemux Forwarding DVD event %s to all pads",
         event_type);
@@ -469,8 +474,6 @@ gst_dvd_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
   gint mpeg_version = *((gint *) info);
 
   if (dvd_demux->mpeg_version != mpeg_version) {
-    if (str->caps)
-      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, mpeg_version,
         "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
@@ -590,8 +593,6 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
         /* Determine the number of channels. */
         channels = ((sample_info >> 8) & 0x7) + 1;
 
-        if (str->caps)
-          gst_caps_unref (str->caps);
         str->caps = gst_caps_new_simple ("audio/x-lpcm",
             "width", G_TYPE_INT, width,
             "rate", G_TYPE_INT, rate,
@@ -611,15 +612,11 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
         break;
 
       case GST_DVD_DEMUX_AUDIO_AC3:
-        if (str->caps)
-          gst_caps_unref (str->caps);
         str->caps = gst_caps_new_simple ("audio/x-ac3", NULL);
         codec = "AC-3 audio";
         break;
 
       case GST_DVD_DEMUX_AUDIO_DTS:
-        if (str->caps)
-          gst_caps_unref (str->caps);
         str->caps = gst_caps_new_simple ("audio/x-dts", NULL);
         codec = "DTS audio";
         break;
@@ -653,8 +650,8 @@ gst_dvd_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
 
         t = g_strdup_printf ("audio-%d-language", stream_nr);
         lang_code =
-            gst_structure_get_string (gst_event_get_structure
-            (dvd_demux->langcodes), t);
+            gst_structure_get_string (gst_event_get_structure (dvd_demux->
+                langcodes), t);
         g_free (t);
       }
 
@@ -723,8 +720,6 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
 
   if (str->type != GST_DVD_DEMUX_SUBP_DVD) {
     /* We need to set new caps for this pad. */
-    if (str->caps)
-      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("video/x-dvd-subpicture", NULL);
 
     if (!gst_pad_set_caps (str->pad, str->caps)) {
@@ -751,8 +746,8 @@ gst_dvd_demux_get_subpicture_stream (GstMPEGDemux * mpeg_demux,
 
         t = g_strdup_printf ("subtitle-%d-language", stream_nr);
         lang_code =
-            gst_structure_get_string (gst_event_get_structure
-            (dvd_demux->langcodes), t);
+            gst_structure_get_string (gst_event_get_structure (dvd_demux->
+                langcodes), t);
         g_free (t);
       }
 
@@ -971,7 +966,7 @@ gst_dvd_demux_combine_flows (GstMPEGDemux * mpegdemux, GstMPEGStream * stream,
   stream->last_flow = flow;
 
   /* if it's success we can return the value right away */
-  if (flow == GST_FLOW_OK)
+  if (GST_FLOW_IS_SUCCESS (flow))
     goto done;
 
   /* any other error that is not-linked can be returned right
@@ -1169,8 +1164,6 @@ gst_dvd_demux_reset (GstDVDDemux * dvd_demux)
 
       gst_element_remove_pad (GST_ELEMENT (dvd_demux),
           dvd_demux->subpicture_stream[i]->pad);
-      if (dvd_demux->subpicture_stream[i]->caps)
-        gst_caps_unref (dvd_demux->subpicture_stream[i]->caps);
       if (dvd_demux->subpicture_stream[i]->tags)
         gst_tag_list_free (dvd_demux->subpicture_stream[i]->tags);
       g_free (dvd_demux->subpicture_stream[i]);
@@ -1187,8 +1180,8 @@ gst_dvd_demux_reset (GstDVDDemux * dvd_demux)
   dvd_demux->mpeg_version = 0;
 
   /* Reset max_gap handling */
-  mpeg_demux->max_gap = MAX_GAP;
-  mpeg_demux->max_gap_tolerance = MAX_GAP_TOLERANCE;
+  mpeg_demux->max_gap = 0.5 * GST_SECOND;
+  mpeg_demux->max_gap_tolerance = 0.05 * GST_SECOND;
 }
 
 static void
@@ -1201,16 +1194,19 @@ gst_dvd_demux_synchronise_pads (GstMPEGDemux * mpeg_demux,
   parent_class->synchronise_pads (mpeg_demux, threshold, new_ts);
 
   for (i = 0; i < GST_DVD_DEMUX_NUM_SUBPICTURE_STREAMS; i++) {
+#ifndef GST_DISABLE_DEBUG
     if (dvd_demux->subpicture_stream[i]) {
       GST_LOG_OBJECT (mpeg_demux, "stream: %d, current: %" GST_TIME_FORMAT
           ", threshold %" GST_TIME_FORMAT, i,
           GST_TIME_ARGS (dvd_demux->subpicture_stream[i]->cur_ts),
           GST_TIME_ARGS (threshold));
-      if (dvd_demux->subpicture_stream[i]->cur_ts < threshold) {
-        DEMUX_CLASS (mpeg_demux)->sync_stream_to_time (mpeg_demux,
-            dvd_demux->subpicture_stream[i], new_ts);
-        dvd_demux->subpicture_stream[i]->cur_ts = new_ts;
-      }
+    }
+#endif
+    if (dvd_demux->subpicture_stream[i]
+        && (dvd_demux->subpicture_stream[i]->cur_ts < threshold)) {
+      DEMUX_CLASS (mpeg_demux)->sync_stream_to_time (mpeg_demux,
+          dvd_demux->subpicture_stream[i], new_ts);
+      dvd_demux->subpicture_stream[i]->cur_ts = new_ts;
     }
   }
 }
