@@ -30,6 +30,14 @@ GST_DEBUG_CATEGORY_STATIC (gstmpegdemux_debug);
 #define PARSE_CLASS(o)  GST_MPEG_PARSE_CLASS (G_OBJECT_GET_CLASS (o))
 #define CLASS(o)  GST_MPEG_DEMUX_CLASS (G_OBJECT_GET_CLASS (o))
 
+/* elementfactory information */
+static GstElementDetails mpeg_demux_details = {
+  "MPEG Demuxer",
+  "Codec/Demuxer",
+  "Demultiplexes MPEG1 and MPEG2 System Streams",
+  "Erik Walthinsen <omega@cse.ogi.edu>\n" "Wim Taymans <wim.taymans@chello.be>"
+};
+
 /* MPEG2Demux signals and args */
 enum
 {
@@ -82,6 +90,8 @@ GST_STATIC_PAD_TEMPLATE ("private_%d",
 GST_BOILERPLATE_FULL (GstMPEGDemux, gst_mpeg_demux, GstMPEGParse,
     GST_TYPE_MPEG_PARSE, _do_init);
 
+static void gst_mpeg_demux_class_init (GstMPEGDemuxClass * klass);
+
 static gboolean gst_mpeg_demux_process_event (GstMPEGParse * mpeg_parse,
     GstEvent * event);
 
@@ -159,10 +169,7 @@ gst_mpeg_demux_base_init (gpointer klass_ptr)
   gst_element_class_add_pad_template (element_class, klass->audio_template);
   gst_element_class_add_pad_template (element_class, klass->private_template);
 
-  gst_element_class_set_details_simple (element_class, "MPEG Demuxer",
-      "Codec/Demuxer",
-      "Demultiplexes MPEG1 and MPEG2 System Streams",
-      "Erik Walthinsen <omega@cse.ogi.edu>, Wim Taymans <wim.taymans@chello.be>");
+  gst_element_class_set_details (element_class, &mpeg_demux_details);
 }
 
 static void
@@ -324,7 +331,6 @@ gst_mpeg_demux_init_stream (GstMPEGDemux * mpeg_demux,
   str->last_flow = GST_FLOW_OK;
   str->buffers_sent = 0;
   str->tags = NULL;
-  str->caps = NULL;
 }
 
 static GstMPEGStream *
@@ -367,8 +373,6 @@ gst_mpeg_demux_get_video_stream (GstMPEGDemux * mpeg_demux,
     GstTagList *list;
 
     /* We need to set new caps for this pad. */
-    if (str->caps)
-      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("video/mpeg",
         "mpegversion", G_TYPE_INT, mpeg_version,
         "systemstream", G_TYPE_BOOLEAN, FALSE, NULL);
@@ -441,8 +445,6 @@ gst_mpeg_demux_get_audio_stream (GstMPEGDemux * mpeg_demux,
     GstTagList *list;
 
     /* We need to set new caps for this pad. */
-    if (str->caps)
-      gst_caps_unref (str->caps);
     str->caps = gst_caps_new_simple ("audio/mpeg",
         "mpegversion", G_TYPE_INT, 1, NULL);
     if (!gst_pad_set_caps (str->pad, str->caps)) {
@@ -497,9 +499,11 @@ static gboolean
 gst_mpeg_demux_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
 {
   GstMPEGDemux *demux = GST_MPEG_DEMUX (mpeg_parse);
+  guint8 *buf;
 
   parent_class->parse_packhead (mpeg_parse, buffer);
 
+  buf = GST_BUFFER_DATA (buffer);
   /* do something useful here */
 
   if (demux->pending_tags) {
@@ -636,19 +640,20 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
   guint16 headerlen;
 
   guint16 packet_length;
-  guint16 STD_buffer_size_bound G_GNUC_UNUSED;
+  gboolean STD_buffer_bound_scale;
+  guint16 STD_buffer_size_bound;
   guint64 dts;
   gint64 pts = -1;
 
   guint16 datalen;
 
   GstMPEGStream *outstream = NULL;
-  guint8 *buf;
+  guint8 *buf, *basebuf;
   gint64 timestamp;
 
   GstFlowReturn ret = GST_FLOW_OK;
 
-  buf = GST_BUFFER_DATA (buffer);
+  basebuf = buf = GST_BUFFER_DATA (buffer);
   id = *(buf + 3);
   buf += 4;
 
@@ -676,7 +681,7 @@ gst_mpeg_demux_parse_packet (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
       case 0x40:
         GST_DEBUG_OBJECT (mpeg_demux, "have STD");
 
-        /* STD_buffer_bound_scale = ((bits & 0x20) == 0x20); */
+        STD_buffer_bound_scale = bits & 0x20;
         STD_buffer_size_bound = ((guint16) (bits & 0x1F)) << 8;
         STD_buffer_size_bound |= *buf++;
 
@@ -953,7 +958,7 @@ gst_mpeg_demux_combine_flows (GstMPEGDemux * demux, GstMPEGStream * stream,
   stream->last_flow = flow;
 
   /* if it's success we can return the value right away */
-  if (flow == GST_FLOW_OK)
+  if (GST_FLOW_IS_SUCCESS (flow))
     goto done;
 
   /* any other error that is not-linked can be returned right
@@ -1029,11 +1034,9 @@ gst_mpeg_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
       outstream->scr_offs = 0;
 
     if (mpeg_demux->index != NULL) {
-      /* Register a new index position.
-       * FIXME: check for keyframes
-       */
+      /* Register a new index position. */
       gst_index_add_association (mpeg_demux->index,
-          outstream->index_id, GST_ASSOCIATION_FLAG_DELTA_UNIT,
+          outstream->index_id, 0,
           GST_FORMAT_BYTES,
           GST_BUFFER_OFFSET (buffer), GST_FORMAT_TIME, timestamp, 0);
     }
@@ -1064,6 +1067,7 @@ gst_mpeg_demux_send_subbuffer (GstMPEGDemux * mpeg_demux,
     GstClockTimeDiff diff;
     guint64 update_time;
 
+    update_time = MIN (timestamp, mpeg_parse->current_segment.stop);
     update_time = MAX (timestamp, mpeg_parse->current_segment.start);
     diff = GST_CLOCK_DIFF (mpeg_parse->current_segment.last_stop, update_time);
     if (diff > GST_SECOND * 2) {
@@ -1330,8 +1334,6 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           gst_event_new_eos ());
       gst_element_remove_pad (GST_ELEMENT (mpeg_demux),
           mpeg_demux->video_stream[i]->pad);
-      if (mpeg_demux->video_stream[i]->caps)
-        gst_caps_unref (mpeg_demux->video_stream[i]->caps);
       g_free (mpeg_demux->video_stream[i]);
       mpeg_demux->video_stream[i] = NULL;
     }
@@ -1343,8 +1345,6 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           mpeg_demux->audio_stream[i]->pad);
       if (mpeg_demux->audio_stream[i]->tags)
         gst_tag_list_free (mpeg_demux->audio_stream[i]->tags);
-      if (mpeg_demux->audio_stream[i]->caps)
-        gst_caps_unref (mpeg_demux->audio_stream[i]->caps);
       g_free (mpeg_demux->audio_stream[i]);
       mpeg_demux->audio_stream[i] = NULL;
     }
@@ -1354,8 +1354,6 @@ gst_mpeg_demux_reset (GstMPEGDemux * mpeg_demux)
           gst_event_new_eos ());
       gst_element_remove_pad (GST_ELEMENT (mpeg_demux),
           mpeg_demux->private_stream[i]->pad);
-      if (mpeg_demux->private_stream[i]->caps)
-        gst_caps_unref (mpeg_demux->private_stream[i]->caps);
       g_free (mpeg_demux->private_stream[i]);
       mpeg_demux->private_stream[i] = NULL;
     }
