@@ -28,12 +28,12 @@
 #  include "config.h"
 #endif
 
-/* FIXME 0.11: suppress warnings for deprecated API such as GStaticRecMutex
- * with newer GLib versions (>= 2.31.0) */
-#define GLIB_DISABLE_DEPRECATION_WARNINGS
-
 #include "rmdemux.h"
+#include "rdtdepay.h"
+#include "rdtmanager.h"
+#include "rtspreal.h"
 #include "rmutils.h"
+#include "rademux.h"
 
 #include <string.h>
 #include <ctype.h>
@@ -46,8 +46,6 @@
 
 #define MAX_FRAGS 256
 
-static const guint8 sipr_subpk_size[4] = { 29, 19, 37, 20 };
-
 typedef struct _GstRMDemuxIndex GstRMDemuxIndex;
 
 struct _GstRMDemuxStream
@@ -81,7 +79,6 @@ struct _GstRMDemuxStream
   guint16 version;
   guint32 extra_data_size;      /* codec_data_length  */
   guint8 *extra_data;           /* extras             */
-  guint32 bitrate;
 
   gboolean needs_descrambling;
   guint subpackets_needed;      /* subpackets needed for descrambling    */
@@ -108,6 +105,13 @@ struct _GstRMDemuxIndex
   GstClockTime timestamp;
 };
 
+static GstElementDetails gst_rmdemux_details = {
+  "RealMedia Demuxer",
+  "Codec/Demuxer",
+  "Demultiplex a RealMedia file into audio and video streams",
+  "David Schleef <ds@schleef.org>"
+};
+
 static GstStaticPadTemplate gst_rmdemux_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -200,16 +204,13 @@ gst_rmdemux_base_init (GstRMDemuxClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rmdemux_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rmdemux_videosrc_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rmdemux_audiosrc_template);
-  gst_element_class_set_details_simple (element_class, "RealMedia Demuxer",
-      "Codec/Demuxer",
-      "Demultiplex a RealMedia file into audio and video streams",
-      "David Schleef <ds@schleef.org>");
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_videosrc_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_audiosrc_template));
+  gst_element_class_set_details (element_class, &gst_rmdemux_details);
 }
 
 static void
@@ -266,8 +267,6 @@ gst_rmdemux_init (GstRMDemux * rmdemux)
   rmdemux->first_ts = GST_CLOCK_TIME_NONE;
   rmdemux->base_ts = GST_CLOCK_TIME_NONE;
   rmdemux->need_newsegment = TRUE;
-
-  gst_rm_utils_run_tests ();
 }
 
 static gboolean
@@ -404,6 +403,9 @@ find_seek_offset_bytes (GstRMDemux * rmdemux, guint target)
   GSList *cur;
   gboolean ret = FALSE;
 
+  if (target < 0)
+    return FALSE;
+
   for (cur = rmdemux->streams; cur; cur = cur->next) {
     GstRMDemuxStream *stream = cur->data;
 
@@ -472,7 +474,7 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
 {
   gboolean validated;
   gboolean ret = TRUE;
-  gboolean flush;
+  gboolean flush, accurate;
   GstFormat format;
   gdouble rate;
   GstSeekFlags flags;
@@ -507,6 +509,7 @@ gst_rmdemux_perform_seek (GstRMDemux * rmdemux, GstEvent * event)
   GST_DEBUG_OBJECT (rmdemux, "seek, rate %g", rate);
 
   flush = flags & GST_SEEK_FLAG_FLUSH;
+  accurate = flags & GST_SEEK_FLAG_ACCURATE;
 
   /* first step is to unlock the streaming thread if it is
    * blocked in a chain call, we do this by starting the flush. */
@@ -609,7 +612,7 @@ done:
   /* streaming can continue now */
   GST_PAD_STREAM_UNLOCK (rmdemux->sinkpad);
 
-  return ret;
+  return TRUE;
 
 error:
   {
@@ -711,11 +714,6 @@ gst_rmdemux_reset (GstRMDemux * rmdemux)
   rmdemux->n_audio_streams = 0;
   rmdemux->n_video_streams = 0;
 
-  if (rmdemux->pending_tags != NULL) {
-    gst_tag_list_free (rmdemux->pending_tags);
-    rmdemux->pending_tags = NULL;
-  }
-
   gst_adapter_clear (rmdemux->adapter);
   rmdemux->state = RMDEMUX_STATE_HEADER;
   rmdemux->have_pads = FALSE;
@@ -924,29 +922,31 @@ need_pause:
     rmdemux->segment_running = FALSE;
     gst_pad_pause_task (rmdemux->sinkpad);
 
-    if (ret == GST_FLOW_UNEXPECTED) {
-      /* perform EOS logic */
-      if (rmdemux->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-        gint64 stop;
+    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
+      if (ret == GST_FLOW_UNEXPECTED) {
+        /* perform EOS logic */
+        if (rmdemux->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+          gint64 stop;
 
-        /* for segment playback we need to post when (in stream time)
-         * we stopped, this is either stop (when set) or the duration. */
-        if ((stop = rmdemux->segment.stop) == -1)
-          stop = rmdemux->segment.duration;
+          /* for segment playback we need to post when (in stream time)
+           * we stopped, this is either stop (when set) or the duration. */
+          if ((stop = rmdemux->segment.stop) == -1)
+            stop = rmdemux->segment.duration;
 
-        GST_LOG_OBJECT (rmdemux, "Sending segment done, at end of segment");
-        gst_element_post_message (GST_ELEMENT (rmdemux),
-            gst_message_new_segment_done (GST_OBJECT (rmdemux),
-                GST_FORMAT_TIME, stop));
+          GST_LOG_OBJECT (rmdemux, "Sending segment done, at end of segment");
+          gst_element_post_message (GST_ELEMENT (rmdemux),
+              gst_message_new_segment_done (GST_OBJECT (rmdemux),
+                  GST_FORMAT_TIME, stop));
+        } else {
+          /* normal playback, send EOS to all linked pads */
+          GST_LOG_OBJECT (rmdemux, "Sending EOS, at end of stream");
+          gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
+        }
       } else {
-        /* normal playback, send EOS to all linked pads */
-        GST_LOG_OBJECT (rmdemux, "Sending EOS, at end of stream");
+        GST_ELEMENT_ERROR (rmdemux, STREAM, FAILED,
+            (NULL), ("stream stopped, reason %s", reason));
         gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
       }
-    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
-      GST_ELEMENT_ERROR (rmdemux, STREAM, FAILED,
-          (NULL), ("stream stopped, reason %s", reason));
-      gst_rmdemux_send_event (rmdemux, gst_event_new_eos ());
     }
     return;
   }
@@ -1287,7 +1287,7 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
 {
   GstCaps *stream_caps = NULL;
   const gchar *codec_tag = NULL;
-  gchar *codec_name = NULL;
+  const gchar *codec_name = NULL;
   int version = 0;
 
   if (stream->subtype == GST_RMDEMUX_STREAM_VIDEO) {
@@ -1301,15 +1301,19 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
 
     switch (stream->fourcc) {
       case GST_RM_VDO_RV10:
+        codec_name = "Real Video 1.0";
         version = 1;
         break;
       case GST_RM_VDO_RV20:
+        codec_name = "Real Video 2.0";
         version = 2;
         break;
       case GST_RM_VDO_RV30:
+        codec_name = "Real Video 3.0";
         version = 3;
         break;
       case GST_RM_VDO_RV40:
+        codec_name = "Real Video 4.0";
         version = 4;
         break;
       default:
@@ -1351,15 +1355,18 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
     switch (stream->fourcc) {
         /* Older RealAudio Codecs */
       case GST_RM_AUD_14_4:
+        codec_name = "Real Audio 14.4kbps";
         version = 1;
         break;
 
       case GST_RM_AUD_28_8:
+        codec_name = "Real Audio 28.8kbps";
         version = 2;
         break;
 
         /* DolbyNet (Dolby AC3, low bitrate) */
       case GST_RM_AUD_DNET:
+        codec_name = "AC-3 audio";
         stream_caps =
             gst_caps_new_simple ("audio/x-ac3", "rate", G_TYPE_INT,
             (int) stream->rate, NULL);
@@ -1371,6 +1378,7 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
         /* MPEG-4 based */
       case GST_RM_AUD_RAAC:
       case GST_RM_AUD_RACP:
+        codec_name = "MPEG4 audio";
         stream_caps =
             gst_caps_new_simple ("audio/mpeg", "mpegversion", G_TYPE_INT,
             (int) 4, "framed", G_TYPE_BOOLEAN, TRUE, NULL);
@@ -1386,14 +1394,13 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
 
         /* Sony ATRAC3 */
       case GST_RM_AUD_ATRC:
+        codec_name = "Sony ATRAC3";
         stream_caps = gst_caps_new_simple ("audio/x-vnd.sony.atrac3", NULL);
-        stream->needs_descrambling = TRUE;
-        stream->subpackets_needed = stream->height;
-        stream->subpackets = NULL;
         break;
 
         /* RealAudio G2 audio */
       case GST_RM_AUD_COOK:
+        codec_name = "Real Audio G2 (Cook)";
         version = 8;
         stream->needs_descrambling = TRUE;
         stream->subpackets_needed = stream->height;
@@ -1402,26 +1409,15 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
 
         /* RALF is lossless */
       case GST_RM_AUD_RALF:
+        /* FIXME: codec_name = */
         GST_DEBUG_OBJECT (rmdemux, "RALF");
         stream_caps = gst_caps_new_simple ("audio/x-ralf-mpeg4-generic", NULL);
         break;
 
+        /* Sipro/ACELP.NET Voice Codec (MIME unknown) */
       case GST_RM_AUD_SIPR:
-
-        if (stream->flavor > 3) {
-          GST_WARNING_OBJECT (rmdemux, "bad SIPR flavor %d, freeing it",
-              stream->flavor);
-          g_free (stream);
-          goto beach;
-        }
-
-        GST_DEBUG_OBJECT (rmdemux, "SIPR");
+        /* FIXME: codec_name = */
         stream_caps = gst_caps_new_simple ("audio/x-sipro", NULL);
-        stream->needs_descrambling = TRUE;
-        stream->subpackets_needed = stream->height;
-        stream->subpackets = NULL;
-        stream->leaf_size = sipr_subpk_size[stream->flavor];
-
         break;
 
       default:
@@ -1447,7 +1443,6 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
           "width", G_TYPE_INT, (int) stream->sample_width,
           "leaf_size", G_TYPE_INT, (int) stream->leaf_size,
           "packet_size", G_TYPE_INT, (int) stream->packet_size,
-          "bitrate", G_TYPE_INT, (int) stream->bitrate,
           "height", G_TYPE_INT, (int) stream->height, NULL);
     }
     rmdemux->n_audio_streams++;
@@ -1500,15 +1495,11 @@ gst_rmdemux_add_stream (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
     gst_pad_set_active (stream->pad, TRUE);
     gst_element_add_pad (GST_ELEMENT_CAST (rmdemux), stream->pad);
 
-    codec_name = gst_pb_utils_get_codec_description (stream_caps);
-
     /* save for later, we must send the tags after the newsegment event */
-    if (codec_tag != NULL && codec_name != NULL) {
-      if (stream->pending_tags == NULL)
-        stream->pending_tags = gst_tag_list_new ();
+    if (codec_name != NULL && codec_tag != NULL) {
+      stream->pending_tags = gst_tag_list_new ();
       gst_tag_list_add (stream->pending_tags, GST_TAG_MERGE_KEEP,
           codec_tag, codec_name, NULL);
-      g_free (codec_name);
     }
   }
 
@@ -1570,8 +1561,6 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const guint8 * data, int length)
   guint str_len = 0;
   int stream_type;
   int offset;
-  guint32 max_bitrate;
-  guint32 avg_bitrate;
 
   stream = g_new0 (GstRMDemuxStream, 1);
 
@@ -1585,26 +1574,8 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const guint8 * data, int length)
   stream->adapter = gst_adapter_new ();
   GST_LOG_OBJECT (rmdemux, "stream_number=%d", stream->id);
 
-  /* parse the bitrates */
-  max_bitrate = RMDEMUX_GUINT32_GET (data + 2);
-  avg_bitrate = RMDEMUX_GUINT32_GET (data + 6);
-  stream->bitrate = avg_bitrate;
-  GST_LOG_OBJECT (rmdemux, "Stream max bitrate=%u", max_bitrate);
-  GST_LOG_OBJECT (rmdemux, "Stream avg bitrate=%u", avg_bitrate);
-  if (max_bitrate != 0) {
-    if (stream->pending_tags == NULL)
-      stream->pending_tags = gst_tag_list_new ();
-    gst_tag_list_add (stream->pending_tags, GST_TAG_MERGE_REPLACE,
-        GST_TAG_MAXIMUM_BITRATE, max_bitrate, NULL);
-  }
-  if (avg_bitrate != 0) {
-    if (stream->pending_tags == NULL)
-      stream->pending_tags = gst_tag_list_new ();
-    gst_tag_list_add (stream->pending_tags, GST_TAG_MERGE_REPLACE,
-        GST_TAG_BITRATE, avg_bitrate, NULL);
-  }
-
   offset = 30;
+  stream_type = GST_RMDEMUX_STREAM_UNKNOWN;
   stream1_type_string = gst_rm_utils_read_string8 (data + offset,
       length - offset, &str_len);
   offset += str_len;
@@ -1748,6 +1719,7 @@ gst_rmdemux_parse_mdpr (GstRMDemux * rmdemux, const guint8 * data, int length)
           break;
         }
       }
+
       /*  14_4, 28_8, cook, dnet, sipr, raac, racp, ralf, atrc */
       GST_DEBUG_OBJECT (rmdemux,
           "Audio stream with rate=%d sample_width=%d n_channels=%d",
@@ -1870,11 +1842,9 @@ gst_rmdemux_parse_cont (GstRMDemux * rmdemux, const guint8 * data, int length)
   GstTagList *tags;
 
   tags = gst_rm_utils_read_tags (data, length, gst_rm_utils_read_string16);
-
-  GST_LOG_OBJECT (rmdemux, "tags: %" GST_PTR_FORMAT, tags);
-
-  rmdemux->pending_tags =
-      gst_tag_list_merge (rmdemux->pending_tags, tags, GST_TAG_MERGE_APPEND);
+  if (tags) {
+    gst_element_found_tags (GST_ELEMENT (rmdemux), tags);
+  }
 }
 
 static GstFlowReturn
@@ -1887,7 +1857,7 @@ gst_rmdemux_combine_flows (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
   stream->last_flow = ret;
 
   /* if it's success we can return the value right away */
-  if (ret == GST_FLOW_OK)
+  if (GST_FLOW_IS_SUCCESS (ret))
     goto done;
 
   /* any other error that is not-linked can be returned right
@@ -1924,9 +1894,10 @@ gst_rmdemux_stream_clear_cached_subpackets (GstRMDemux * rmdemux,
 }
 
 static GstFlowReturn
-gst_rmdemux_descramble_audio (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
+gst_rmdemux_descramble_cook_audio (GstRMDemux * rmdemux,
+    GstRMDemuxStream * stream)
 {
-  GstFlowReturn ret = GST_FLOW_ERROR;
+  GstFlowReturn ret;
   GstBuffer *outbuf;
   guint packet_size = stream->packet_size;
   guint height = stream->subpackets->len;
@@ -1938,8 +1909,12 @@ gst_rmdemux_descramble_audio (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
   GST_LOG ("packet_size = %u, leaf_size = %u, height= %u", packet_size,
       leaf_size, height);
 
-  outbuf = gst_buffer_new_and_alloc (height * packet_size);
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (stream->pad));
+  ret = gst_pad_alloc_buffer_and_set_caps (stream->pad,
+      GST_BUFFER_OFFSET_NONE, height * packet_size,
+      GST_PAD_CAPS (stream->pad), &outbuf);
+
+  if (ret != GST_FLOW_OK)
+    goto done;
 
   for (p = 0; p < height; ++p) {
     GstBuffer *b = g_ptr_array_index (stream->subpackets, p);
@@ -1980,6 +1955,8 @@ gst_rmdemux_descramble_audio (GstRMDemux * rmdemux, GstRMDemuxStream * stream)
 
   gst_buffer_unref (outbuf);
 
+done:
+
   gst_rmdemux_stream_clear_cached_subpackets (rmdemux, stream);
 
   return ret;
@@ -2012,6 +1989,7 @@ gst_rmdemux_descramble_mp4a_audio (GstRMDemux * rmdemux,
   GstBuffer *buf, *outbuf;
   guint frames, index, i;
   guint8 *data;
+  guint size;
   GstClockTime timestamp;
 
   res = GST_FLOW_OK;
@@ -2021,6 +1999,7 @@ gst_rmdemux_descramble_mp4a_audio (GstRMDemux * rmdemux,
   g_ptr_array_set_size (stream->subpackets, 0);
 
   data = GST_BUFFER_DATA (buf);
+  size = GST_BUFFER_SIZE (buf);
   timestamp = GST_BUFFER_TIMESTAMP (buf);
 
   frames = (data[1] & 0xf0) >> 4;
@@ -2048,52 +2027,6 @@ gst_rmdemux_descramble_mp4a_audio (GstRMDemux * rmdemux,
   return res;
 }
 
-static GstFlowReturn
-gst_rmdemux_descramble_sipr_audio (GstRMDemux * rmdemux,
-    GstRMDemuxStream * stream)
-{
-  GstFlowReturn ret;
-  GstBuffer *outbuf;
-  guint packet_size = stream->packet_size;
-  guint height = stream->subpackets->len;
-  guint p;
-
-  g_assert (stream->height == height);
-
-  GST_LOG ("packet_size = %u, leaf_size = %u, height= %u", packet_size,
-      stream->leaf_size, height);
-
-  outbuf = gst_buffer_new_and_alloc (height * packet_size);
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (stream->pad));
-
-  for (p = 0; p < height; ++p) {
-    GstBuffer *b = g_ptr_array_index (stream->subpackets, p);
-    guint8 *b_data = GST_BUFFER_DATA (b);
-
-    if (p == 0)
-      GST_BUFFER_TIMESTAMP (outbuf) = GST_BUFFER_TIMESTAMP (b);
-
-    memcpy (GST_BUFFER_DATA (outbuf) + packet_size * p, b_data, packet_size);
-  }
-
-  GST_LOG_OBJECT (rmdemux, "pushing buffer timestamp %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuf)));
-
-  if (stream->discont) {
-    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_DISCONT);
-    stream->discont = FALSE;
-  }
-
-  outbuf = gst_rm_utils_descramble_sipr_buffer (outbuf);
-
-  gst_buffer_set_caps (outbuf, GST_PAD_CAPS (stream->pad));
-  ret = gst_pad_push (stream->pad, outbuf);
-
-  gst_rmdemux_stream_clear_cached_subpackets (rmdemux, stream);
-
-  return ret;
-}
-
 static GstFlowReturn
 gst_rmdemux_handle_scrambled_packet (GstRMDemux * rmdemux,
     GstRMDemuxStream * stream, GstBuffer * buf, gboolean keyframe)
@@ -2122,16 +2055,12 @@ gst_rmdemux_handle_scrambled_packet (GstRMDemux * rmdemux,
       ret = gst_rmdemux_descramble_dnet_audio (rmdemux, stream);
       break;
     case GST_RM_AUD_COOK:
-    case GST_RM_AUD_ATRC:
-      ret = gst_rmdemux_descramble_audio (rmdemux, stream);
+      ret = gst_rmdemux_descramble_cook_audio (rmdemux, stream);
       break;
     case GST_RM_AUD_RAAC:
     case GST_RM_AUD_RACP:
       ret = gst_rmdemux_descramble_mp4a_audio (rmdemux, stream);
       break;
-    case GST_RM_AUD_SIPR:
-      ret = gst_rmdemux_descramble_sipr_audio (rmdemux, stream);
-      break;
     default:
       g_assert_not_reached ();
   }
@@ -2458,10 +2387,6 @@ gst_rmdemux_parse_video_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
         stream->discont = FALSE;
       }
 
-      if (!key) {
-        GST_BUFFER_FLAG_SET (out, GST_BUFFER_FLAG_DELTA_UNIT);
-      }
-
       ret = gst_pad_push (stream->pad, out);
       ret = gst_rmdemux_combine_flows (rmdemux, stream, ret);
       if (ret != GST_FLOW_OK)
@@ -2501,7 +2426,7 @@ gst_rmdemux_parse_audio_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
     GstBuffer * in, guint offset, guint16 version,
     GstClockTime timestamp, gboolean key)
 {
-  GstFlowReturn ret;
+  GstFlowReturn ret, cret;
   GstBuffer *buffer;
   const guint8 *data;
   guint size;
@@ -2509,8 +2434,12 @@ gst_rmdemux_parse_audio_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
   data = GST_BUFFER_DATA (in) + offset;
   size = GST_BUFFER_SIZE (in) - offset;
 
-  buffer = gst_buffer_new_and_alloc (size);
-  gst_buffer_set_caps (buffer, GST_PAD_CAPS (stream->pad));
+  ret = gst_pad_alloc_buffer_and_set_caps (stream->pad,
+      GST_BUFFER_OFFSET_NONE, size, GST_PAD_CAPS (stream->pad), &buffer);
+
+  cret = gst_rmdemux_combine_flows (rmdemux, stream, ret);
+  if (ret != GST_FLOW_OK)
+    goto alloc_failed;
 
   memcpy (GST_BUFFER_DATA (buffer), (guint8 *) data, size);
 
@@ -2540,10 +2469,14 @@ gst_rmdemux_parse_audio_packet (GstRMDemux * rmdemux, GstRMDemuxStream * stream,
     }
     ret = gst_pad_push (stream->pad, buffer);
   }
-
-  gst_buffer_unref (in);
-
   return ret;
+
+  /* ERRORS */
+alloc_failed:
+  {
+    GST_DEBUG_OBJECT (rmdemux, "pad alloc returned %d", ret);
+    return cret;
+  }
 }
 
 static GstFlowReturn
@@ -2589,16 +2522,19 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, GstBuffer * in, guint16 version)
   data += (2 + 4);
   size -= (2 + 4);
 
-  /* get flags */
-  flags = GST_READ_UINT8 (data + 1);
-
-  data += 2;
-  size -= 2;
-
-  /* version 1 has an extra byte */
-  if (version == 1) {
-    data += 1;
-    size -= 1;
+  /* skip other stuff */
+  if (version == 0) {
+    /* uint8 packet_group */
+    /* uint8 flags        */
+    flags = GST_READ_UINT8 (data + 1);
+    data += 2;
+    size -= 2;
+  } else {
+    /* uint16 asm_rule */
+    /* uint8 asm_flags */
+    flags = GST_READ_UINT8 (data + 2);
+    data += 3;
+    size -= 3;
   }
   key = (flags & 0x02) != 0;
   GST_DEBUG_OBJECT (rmdemux, "flags %d, Keyframe %d", flags, key);
@@ -2615,11 +2551,6 @@ gst_rmdemux_parse_packet (GstRMDemux * rmdemux, GstBuffer * in, guint16 version)
 
     gst_rmdemux_send_event (rmdemux, event);
     rmdemux->need_newsegment = FALSE;
-
-    if (rmdemux->pending_tags != NULL) {
-      gst_element_found_tags (GST_ELEMENT (rmdemux), rmdemux->pending_tags);
-      rmdemux->pending_tags = NULL;
-    }
   }
 
   if (stream->pending_tags != NULL) {
@@ -2663,9 +2594,36 @@ unknown_stream:
   }
 }
 
-gboolean
+static gboolean
 gst_rmdemux_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rmdemux",
-      GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
+      GST_RANK_PRIMARY, GST_TYPE_RMDEMUX) &&
+      gst_element_register (plugin, "rademux",
+      GST_RANK_SECONDARY, GST_TYPE_REAL_AUDIO_DEMUX);
 }
+
+
+static gboolean
+plugin_init (GstPlugin * plugin)
+{
+  if (!gst_rmdemux_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rdt_depay_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rdt_manager_plugin_init (plugin))
+    return FALSE;
+
+  if (!gst_rtsp_real_plugin_init (plugin))
+    return FALSE;
+
+  return TRUE;
+}
+
+GST_PLUGIN_DEFINE (GST_VERSION_MAJOR,
+    GST_VERSION_MINOR,
+    "realmedia",
+    "RealMedia demuxer and depayloader",
+    plugin_init, VERSION, "LGPL", GST_PACKAGE_NAME, GST_PACKAGE_ORIGIN);
